#ifndef __SINGLETON__
#define __SINGLETON__
///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: Singleton.hpp $
// Version        : $Revision: 1 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 24/07/99 14:42 $
//
// Notes          : A template base class for classes that only have one instance.
//
// Modifications  :
//
// $Log: /JetByteTools/C++ Tools/Singleton.hpp $
// 
// 1     24/07/99 14:42 Len
// 
//    Rev 1.0   May 18 1998 07:34:24   Len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1998 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// Templates defined in this file...
///////////////////////////////////////////////////////////////////////////////

template <class T> class TSingleton;

///////////////////////////////////////////////////////////////////////////////
// TSingleton
///////////////////////////////////////////////////////////////////////////////

template <class T> class TSingleton
{
   public :

      // Construction and destruction

      virtual ~TSingleton();

      // Access to the one instance of the object

      static T &TheObject(); 

   protected :

      // We can only be created in our access function...

      TSingleton();
};

///////////////////////////////////////////////////////////////////////////////
// Construction and destriction
///////////////////////////////////////////////////////////////////////////////

template <class T> TSingleton<T>::TSingleton()
{
   // We can only be created in our access function...
}


template <class T> TSingleton<T>::~TSingleton()
{
   // Nothing to do, we just need to be virtual
}

///////////////////////////////////////////////////////////////////////////////
// Access to the one instance of the object
///////////////////////////////////////////////////////////////////////////////

template <class T> T &TSingleton<T>::TheObject()
{
   static T theObject;

   return theObject;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

#endif // __SINGLETON__

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
