///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: ComExeServer.cpp $
// Version        : $Revision: 3 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 6/03/00 7:42 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/COMTools/ComExeServer.cpp $
// 
// 3     6/03/00 7:42 Len
// Exclude from the build unless we're building for NT4 / 95+DCOM only.
// 
// 2     28/07/99 20:06 Len
// Pragma to remove warning C4711
// 
// 1     22/07/99 23:56 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 23:08:20   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#if (_WIN32_WINNT >= 0x0400 ) || defined(_WIN32_DCOM)

#include "ComExeServer.hpp"
#include "ComException.hpp"
#include "RegistryKey.hpp"
#include "OleString.hpp"
#include "LocalSecurityPolicy.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CComExeServer
///////////////////////////////////////////////////////////////////////////////

HANDLE CComExeServer::s_hEventDone = CreateEvent(0, TRUE, FALSE, 0);

HINSTANCE CComExeServer::s_hInstance = (HINSTANCE)INVALID_HANDLE_VALUE;

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CComExeServer::CComExeServer()
{
}

CComExeServer::~CComExeServer()
{
}

///////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////

bool CComExeServer::InitInstance(LPCTSTR /*lpCmdLine*/)
{
	// Do nothing by default

	return true;
}

void CComExeServer::ShutdownInstance()
{
	// Do nothing by default
}

void CComExeServer::RegisterServer(LPCTSTR /*lpCmdLine*/)
{
	LPCTSTR pClsidAsString = _T("{3BFEE261-7665-11d2-802D-0020186676FB}");

	try
	{
		CRegistryKey key = CRegistryKey(HKEY_CLASSES_ROOT, _T("CLSID")).CreateOrOpenKey(pClsidAsString);

		key.SetValue(_T(""), _T("ChatSession"));

		// Do this in the appid section?
		key.SetValue(_T("AppId"), pClsidAsString);

		key = key.CreateOrOpenKey(_T("LocalServer32"));

		TCHAR szFileName[MAX_PATH * sizeof(TCHAR)];
		GetModuleFileName(s_hInstance, szFileName, MAX_PATH);

		key.SetValue(_T(""), szFileName);

		// App Id

		LPCTSTR pAppIDAsString = pClsidAsString;

		CRegistryKey AppIDkey = CRegistryKey(HKEY_CLASSES_ROOT, _T("AppID")).CreateOrOpenKey(_T("Server2.exe"));

		AppIDkey.SetValue(_T("AppId"), pAppIDAsString);

		// Now the ones under our appid...

		AppIDkey = CRegistryKey(HKEY_CLASSES_ROOT, _T("AppID")).CreateOrOpenKey(pAppIDAsString);

		AppIDkey.SetValue(_T(""), _T("ChatSession Server"));

		// Run as requies the password to be set too

		LPCTSTR pRunAsUser = _T("TWIN-PII\\len");

		LPCTSTR pPassword = _T("");

		SetRunAs(pAppIDAsString, pRunAsUser, pPassword);

		// Stream out the contents of the launch acl

		//AppIDkey.SetValue(_T("LaunchPermission"), (LPBYTE)"Dummy", sizeof("Dummy"));

		//AppIDkey.SetValue(_T("AccessPermission"), (LPBYTE)"Dummy", sizeof("Dummy"));

		// These are our own special extra things...

		AppIDkey.SetValue(_T("Chat Admins Group"), _T("TWIN-PII\\ChatAdmin"));

		AppIDkey.SetValue(_T("Chat Users Group"), _T("TWIN-PII\\ChatUsers"));
	}
	catch(CWin32Exception &e)
	{
		e.MessageBox();
	}
}

void CComExeServer::SetRunAs(
	LPCTSTR pAppId, 
	LPCTSTR pUser,
	LPCTSTR pPassword)
{
	CRegistryKey AppIDkey = CRegistryKey(HKEY_CLASSES_ROOT, _T("AppID")).CreateOrOpenKey(pAppId);

	AppIDkey.SetValue(_T("RunAs"), pUser);

	// Now store the password away as a LSA secret...

	CLocalSecurityPolicy lsa;

	TCHAR key[200];

	_tcscpy(key, _T("SCM:"));
	_tcscat(key, COleString(pAppId));

	lsa.StorePrivateData(key, pPassword);

	lsa.AddAccountRights(pUser, _T("SeBatchLogonRight"));
}

void CComExeServer::UnregisterServer(LPCTSTR /*lpCmdLine*/)
{

}

void CComExeServer::RegisterClassObject(
	const CLSID &clsid,
	IUnknown *pUnknown)
{
	HRESULT hr = CoRegisterClassObject(
		clsid,
		pUnknown,
      CLSCTX_LOCAL_SERVER,
      REGCLS_SUSPENDED | REGCLS_MULTIPLEUSE,
		&m_dwReg);

	CComException::ThrowOnFailure(
			_T("CComExeServer::RegisterClassObject() - CoRegisterClassObject()"),
			hr);
}

void CComExeServer::RevokeClassObject()
{
	HRESULT hr = CoRevokeClassObject(m_dwReg);

	CComException::ThrowOnFailure(
			_T("CComExeServer::RevokeClassObject() - CoRevokeClassObject()"),
			hr);
}

///////////////////////////////////////////////////////////////////////////////
// Application level static functions...
///////////////////////////////////////////////////////////////////////////////

void CComExeServer::Run(HINSTANCE hInstance, LPCTSTR lpCmdLine)
{
	if (Initialise(hInstance, lpCmdLine))
   {
		HRESULT hr = CoInitializeEx(0, COINIT_MULTITHREADED);
 
		CComException::ThrowOnFailure(
			_T("CComExeServer::Run() - CoInitializeEx()"), 
			hr);

		RegisterClassObjects();

		// We're all set, lets go..

		hr = CoResumeClassObjects();
       
		if (SUCCEEDED(hr))
		{
          WaitForSingleObject(s_hEventDone, INFINITE);
		}
		else
		{
			CComException(
				_T("CComExeServer::Run() - CoResumeClassObjects()"), 
				hr).MessageBox();
		}

		RevokeClassObjects();

		CoUninitialize();

		Shutdown();
	}
}

void CComExeServer::RegisterClassObjects()
{
	for (Iterator it = Begin(); it != End(); ++it)
	{
		it->RegisterClassObject();
	}
}

void CComExeServer::RevokeClassObjects()
{
   for (Iterator it = Begin(); it != End(); ++it)
   {
      it->RevokeClassObject();
   }
}


ULONG CComExeServer::Lock()
{
	return CoAddRefServerProcess();
}

ULONG CComExeServer::Unlock()
{
	ULONG count = CoReleaseServerProcess();
	
	if (count == 0)
	{
		SetEvent(s_hEventDone);
	}

	return count;
}


bool CComExeServer::Initialise(HINSTANCE hInstance, LPCTSTR lpCmdLine)
{
   bool ok = false;

   s_hInstance = hInstance;

   if (ParseCommandLine(lpCmdLine))
   {
	   Iterator it = Begin();
	
		do 
		{
			ok = it->InitInstance(lpCmdLine);
		}
		while (ok && ++it != End());
   }

   return ok;
}

void CComExeServer::Shutdown()
{
	Iterator it = Begin();

	do 
	{
		it->ShutdownInstance();
	}
	while (++it != End());
}

bool CComExeServer::ParseCommandLine(LPCTSTR lpCmdLine)
{
	bool ok = true;

	if (_tcsstr(lpCmdLine, _T("UnregServer")) != 0 || 
		 _tcsstr(lpCmdLine, _T("unregserver")) != 0)
	{
		UnregisterServers(lpCmdLine);
		ok = false;
	}
	else if (_tcsstr(lpCmdLine, _T("RegServer")) != 0 ||
				_tcsstr(lpCmdLine, _T("regserver")) != 0)
	{
		RegisterServers(lpCmdLine);
		ok = false;
	}

	return ok;
}

void CComExeServer::RegisterServers(LPCTSTR lpCmdLine)
{
   for (Iterator it = Begin(); it != End(); ++it)
   {
      it->RegisterServer(lpCmdLine);
   }
}

void CComExeServer::UnregisterServers(LPCTSTR lpCmdLine)
{
   for (Iterator it = Begin(); it != End(); ++it)
   {
      it->UnregisterServer(lpCmdLine);
   }
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

#endif // (_WIN32_WINNT >= 0x0400 ) || defined(_WIN32_DCOM)

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
