///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: ComGUID.cpp $
// Version        : $Revision: 4 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 12/06/01 19:17 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/COMTools/ComGUID.cpp $
// 
// 4     12/06/01 19:17 Len
// Open registry keys with KEY_READ rather than KEY_ALL_ACCESS
// 
// 3     29/11/99 0:37 Len
// More static members for general GUID manipulation.
// 
// 2     28/07/99 20:06 Len
// Pragma to remove warning C4711
// 
// 1     22/07/99 23:56 Len
// Moved to source safe
// 
//    Rev 1.4   May 13 1999 23:09:04   len
// 
//    Rev 1.3   Oct 25 1998 21:04:34   len
// Added GetDescription() as an alternative to Description()...
// 
//    Rev 1.2   Oct 25 1998 21:01:06   len
// 
//    Rev 1.1   Jun 06 1998 07:55:04   Len
// 
//    Rev 1.0   May 18 1998 07:34:44   Len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1998 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "ComGUID.hpp"
#include "OleString.hpp"
#include "ComException.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CComGUID
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CComGUID::CComGUID(const GUID &guid)
   :  m_guid(guid), 
      m_pGuid(0),
      m_pDesc(0),
      m_pRealDesc(0),
      m_regKey(CRegistryKey(HKEY_CLASSES_ROOT).OpenKey(_T("CLSID"),KEY_READ).OpenKey(AsString(),KEY_READ))
{

}

CComGUID::CComGUID(const CComGUID &rhs)
   :  m_guid(rhs.m_guid),
      m_pGuid(0), 
      m_pDesc(0),
      m_pRealDesc(0),
      m_regKey(rhs.m_regKey)
{
   
}

CComGUID::CComGUID(LPCTSTR pGuidAsString)
   :  m_guid(FromString(pGuidAsString)),
      m_pGuid(0), 
      m_pDesc(0),
      m_pRealDesc(0),
      m_regKey(CRegistryKey(HKEY_CLASSES_ROOT).OpenKey(_T("CLSID"),KEY_READ).OpenKey(AsString(),KEY_READ))
{
   
}

CComGUID::~CComGUID()
{
   free((void*)m_pGuid);
   free((void*)m_pRealDesc);
   m_pDesc = 0;
}

CComGUID &CComGUID::operator=(const CComGUID &rhs)
{
   if (this != &rhs)
   {
      m_guid = rhs.m_guid;
      free((void*)m_pGuid);
      free((void*)m_pRealDesc);
      m_pGuid = 0;
      m_pDesc = 0;
      m_pRealDesc = 0,

      m_regKey = rhs.m_regKey;
   }

   return *this;
}


LPTSTR CComGUID::AsString() const
{
   if (!m_pGuid)
   {
      m_pGuid = AsString(m_guid);
   }

   return m_pGuid;
}

LPTSTR CComGUID::GetDescription() const
{
	return Description();
}

LPTSTR CComGUID::Description() const
{
   if (!m_pDesc)
   {
      if (m_regKey.QueryValue(NULL, &m_pRealDesc))
      {
         if (*m_pRealDesc == _T('\0'))
         {
            free((void*)m_pRealDesc);
            m_pRealDesc = 0;  
         }
         else
         {
            m_pDesc = m_pRealDesc;
         }
      }

      if (!m_pDesc)
      {
         m_pDesc = AsString();
      }

   }

   return m_pDesc;
}

///////////////////////////////////////////////////////////////////////////////
// Private helpers
///////////////////////////////////////////////////////////////////////////////

GUID CComGUID::FromString(LPCTSTR pGuidAsString) 
{
   GUID guid;

   HRESULT hr = IIDFromString((LPOLESTR)_ONCC(pGuidAsString), &guid);

   JetByteTools::CComException::ThrowOnFailure(_T("CComGUID::FromString() - failed to convert string to GUID"), hr);

   return guid;

}

LPTSTR CComGUID::AsString(const GUID &guid)
{
   LPOLESTR lpGUIDString;

   HRESULT hr = StringFromIID(guid, &lpGUIDString);
   
   JetByteTools::CComException::ThrowOnFailure(_T("CComGUID::FromString() - failed to convert string to GUID"), hr);
   
   LPTSTR pGuidAsString = COleString(lpGUIDString).DuplicateAs_tchar();

   // TODO use a smart pointer here to avoid this potential memory leak
   CoTaskMemFree(lpGUIDString);

   return pGuidAsString;
}

GUID CComGUID::GenerateGuid()
{
   GUID guid;

	HRESULT hr = CoCreateGuid(&guid);

	JetByteTools::CComException::ThrowOnFailure(_T("CComGUID::GenerateGuid() - failed to generate GUID"), hr);

	return guid;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
