///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: ComMessageFilter.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 28/07/99 20:06 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/COM Tools/ComMessageFilter.cpp $
// 
// 2     28/07/99 20:06 Len
// Pragma to remove warning C4711
// 
// 1     22/07/99 23:56 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 23:09:24   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "ComMessageFilter.hpp"

#include "ComException.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CComMessageFilter
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CComMessageFilter::CComMessageFilter()
{
	HRESULT hr = CoRegisterMessageFilter(this, &m_pOldMessageFilter);

	if (S_OK != hr)
	{
		throw Exception(
			_T("CComMessageFilter::CComMessageFilter() - CoRegisterMessageFilter() Failed"), 
			hr);
	}
}

CComMessageFilter::~CComMessageFilter()
{
	CoRegisterMessageFilter(m_pOldMessageFilter, 0);	
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

STDMETHODIMP CComMessageFilter::QueryInterface(REFIID riid, void**ppv)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown ||
		 riid == IID_IMessageFilter)
	{
		*ppv = static_cast<IMessageFilter*>(this);
		reinterpret_cast<IUnknown*>(*ppv)->AddRef();
	}
	else
	{
		*ppv = 0;
		hr = E_NOINTERFACE;
	}
	
	return hr;
}

STDMETHODIMP_(ULONG) CComMessageFilter::AddRef(void)
{
	return 2;
}

STDMETHODIMP_(ULONG) CComMessageFilter::Release(void)
{
	return 1;
}

///////////////////////////////////////////////////////////////////////////////
// This is who we are...
///////////////////////////////////////////////////////////////////////////////

STDMETHODIMP_(DWORD) CComMessageFilter::HandleInComingCall(
	DWORD /* dwCallType */, 
	HTASK /* htaskCaller */,
	DWORD /* dwTickCount */, 
	LPINTERFACEINFO /*lpInterfaceInfo*/)
{
	return SERVERCALL_ISHANDLED;
}

STDMETHODIMP_(DWORD) CComMessageFilter::RetryRejectedCall(
	HTASK /* htaskCaller */,
	DWORD /* dwTickCount */,
	DWORD /* dwRejectType */)
{
	return 0;		 // retry now
}

STDMETHODIMP_(DWORD) CComMessageFilter::MessagePending(
	HTASK /* htaskCaller */,
	DWORD /* dwTickCount */, 
	DWORD /* dwPendingType */)
{
	MSG msg;

	// allow application to process pending message
	if (::PeekMessage(&msg, NULL, NULL, NULL, PM_NOREMOVE|PM_NOYIELD))
	{
		DispatchMessage(&msg);
	}

	return PENDINGMSG_WAITNOPROCESS;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
