#ifndef __T_COM_ENUMERATOR__
#define __T_COM_ENUMERATOR__
///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: TComEnumerator.hpp $
// Version        : $Revision: 1 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 22/07/99 23:56 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/COM Tools/TComEnumerator.hpp $
// 
// 1     22/07/99 23:56 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 23:20:42   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {
 
///////////////////////////////////////////////////////////////////////////////
// Classes defined in this file...
///////////////////////////////////////////////////////////////////////////////

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
class TComEnumerator;

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// TComEnumerator
///////////////////////////////////////////////////////////////////////////////
template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
class TComEnumerator : public Interface
{
   public :

      TComEnumerator(Collection *pCollection);				// We now own pCollection
      TComEnumerator(const Collection &collection);		// We take a copy of collection

		~TComEnumerator();

		// IUnknown methods
		STDMETHODIMP QueryInterface(REFIID riid, void **ppv);
		STDMETHODIMP_(ULONG) AddRef(void);
		STDMETHODIMP_(ULONG) Release(void);

		// IEnumXXXX methods
		STDMETHODIMP Next(
			ULONG cElems, 
			Type **rgElems, 
			ULONG *pcFetched);

		STDMETHODIMP Skip(ULONG cElems);
		STDMETHODIMP Reset(void);
		STDMETHODIMP Clone(Interface **ppEnum);
	
	private :

		TComEnumerator(TComEnumerator *pClonedFrom);
		Collection &GetCollection() const;

		Collection *m_pCollection;

		TComEnumerator *m_pClonedFrom;

		LONG m_cRef;
		Collection::iterator m_cursor;
};

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
TComEnumerator<Interface, pIID, Collection, Type, Traits>::TComEnumerator(
	Collection *pCollection) 
	:	m_cRef(0), 
		m_pCollection(pCollection),
		m_pClonedFrom(0),
		m_cursor(GetCollection().begin())
{

}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
TComEnumerator<Interface, pIID, Collection, Type, Traits>::TComEnumerator(
	const Collection &collection) 
	:	m_cRef(0), 
		m_pCollection(Traits::CopyCollection(collection)),
		m_pClonedFrom(0),
		m_cursor(m_pCollection->begin())
{

}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
TComEnumerator<Interface, pIID, Collection, Type, Traits>::TComEnumerator(
	TComEnumerator *pClonedFrom)
	:	m_cRef(0), 
		m_pCollection(0),
		m_pClonedFrom(pClonedFrom),
		m_cursor(pClonedFrom->m_cursor)
{
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
TComEnumerator<Interface, pIID, Collection, Type, Traits>::~TComEnumerator()
{
	Traits::DeleteCollection(m_pCollection);

	if (m_pClonedFrom)
	{
		m_pClonedFrom->Release();
	}
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP TComEnumerator<Interface, pIID, Collection, Type, Traits>::QueryInterface(REFIID riid, void **ppv)
{
	HRESULT hr = E_NOINTERFACE;

	if (!ppv)
	{
		hr = E_INVALIDARG;
	}
	else if (riid == IID_IUnknown ||
				riid == *pIID)
	{
		*ppv = static_cast<Interface*>(this);
		reinterpret_cast<IUnknown*>(*ppv)->AddRef();

		hr = S_OK;
	}
	else
	{
		*ppv = 0;
	}
	
	return hr;
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP_(ULONG) TComEnumerator<Interface, pIID, Collection, Type, Traits>::AddRef(void)
{
	// module lock
	return InterlockedIncrement(&m_cRef);
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP_(ULONG) TComEnumerator<Interface, pIID, Collection, Type, Traits>::Release(void)
{
	LONG ref = InterlockedDecrement(&m_cRef);

	if (ref == 0)
	{
		delete this;
	}
	// module unlock

	return ref;
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP TComEnumerator<Interface, pIID, Collection, Type, Traits>::Next(
	ULONG cElems, 
	Type **rgElems, 
	ULONG *pcFetched)
{
	HRESULT hr = S_OK;

	if ((cElems > 1 && !pcFetched) || !rgElems)
	{
		hr = E_INVALIDARG;
	}
	else
	{	
//		Locker lock(&m_csLock);

		ULONG cActual = 0;
		Collection::iterator end = GetCollection().end();
		Collection::iterator originalCursor = m_cursor;
		bool ok = true;

		while (cActual < cElems &&
				 m_cursor != end && 
				 ok)
		{
			if (Traits::Copy(*m_cursor, &rgElems[cActual]))
			{
				m_cursor++;
				cActual++;
			}
			else
			{
				// Copy failed...

				while (cActual > 0)
				{
					cActual--;
					Traits::Delete(rgElems[cActual]);
					rgElems[cActual] = 0;
				}
				
				m_cursor = originalCursor;
				ok = false;
			}
		}

		if (pcFetched)
		{
        *pcFetched = cActual;
		}

		if (cElems != cActual)
		{
			hr = S_FALSE;
		}
	}

	return hr;
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP TComEnumerator<Interface, pIID, Collection, Type, Traits>::Skip(ULONG cElems)
{
//		Locker lock(&m_csLock);
	Collection::iterator end = GetCollection().end();

	ULONG cSkipped = 0;

	while(cSkipped < cElems &&
		   m_cursor != end)
	{
		m_cursor++;
		cSkipped++;
	}
	
	return ((cSkipped == cElems) ? S_OK : S_FALSE);
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP TComEnumerator<Interface, pIID, Collection, Type, Traits>::Reset(void)
{
//	Locker lock(&m_csLock);
	m_cursor = GetCollection().begin();

	return S_OK;
}

template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
STDMETHODIMP TComEnumerator<Interface, pIID, Collection, Type, Traits>::Clone(Interface **ppEnum)
{
	HRESULT hr = E_OUTOFMEMORY;

	if (!ppEnum)
	{
		hr = E_INVALIDARG;
	}
	else
	{
		TComEnumerator *pClonedFrom = m_pClonedFrom;

		if (!pClonedFrom)
		{
			pClonedFrom = this;
		}

		try
		{
			*ppEnum = new TComEnumerator(pClonedFrom);

			if (ppEnum)
			{
				(*ppEnum)->AddRef();			
				pClonedFrom->AddRef();
				hr = S_OK;
			}
		}
		catch(...)
		{
			// Nothing to do...
		}
	}

	return hr;
}
	
template <class Interface, const GUID *pIID, class Collection, class Type, class Traits> 
Collection &TComEnumerator<Interface, pIID, Collection, Type, Traits>::GetCollection() const
{
	Collection *pCollection = 0;

	if (m_pClonedFrom)
	{
		pCollection = &m_pClonedFrom->GetCollection();
	}
	else
	{
		pCollection = m_pCollection;
	}

	return *pCollection;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

#endif // __T_COM_ENUMERATOR__

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
