///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMCall.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 31/07/99 10:02 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMCall.cpp $
// 
// 2     31/07/99 10:02 Len
// Pragma to remove warning C4711
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:24:56   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmCall.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmCall
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmCall::CNmCall(
	INmCall *pICall,
	const bool bAutoConnect /* = false */)
	:	m_pICall(SafeAddRef(pICall))
{
	if (0 == m_pICall)
	{
		throw Exception(_T("CNmCall::CNmCall()"), E_INVALIDARG); 
	}

	if (bAutoConnect)
	{
		Connect();
	}
}

CNmCall::CNmCall(const CNmCall &rhs)
	:	m_pICall(SafeAddRef(rhs.m_pICall))
{
	if (0 == m_pICall)
	{
		throw Exception(_T("CNmCall::CNmCall()"), E_INVALIDARG); 
	}	

	if (rhs.Connected())
	{
		Connect();
	}
}

CNmCall::~CNmCall()
{
	m_pICall->Release();
	m_pICall = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Implement CNmCall
///////////////////////////////////////////////////////////////////////////////

void CNmCall::Accept()
{
	HRESULT hr = m_pICall->Accept();

	if (S_OK != hr)
	{
		throw Exception(_T("CNmCall::Accept()"), hr);
	}
}

void CNmCall::Cancel()
{
	HRESULT hr = m_pICall->Cancel();

	if (S_OK != hr)
	{
		throw Exception(_T("CNmCall::Cancel()"), hr);
	}
}

// GetAddress()

INmConference *CNmCall::GetConference()
{
	INmConference *pConference = 0;

	HRESULT hr = m_pICall->GetConference(
		&pConference);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmCall::GetConference()"), hr);
	}

	return pConference;
}

LPCTSTR CNmCall::GetName()
{
	BSTR name;

	HRESULT hr = m_pICall->GetName(&name);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmCall::GetName()"), hr);
	}

	COleString string(name);

	LPTSTR pName = string.DuplicateAs_tchar();
	
	SysFreeString(name);

	return pName;
}

//GetState  
//GetUserData  

bool CNmCall::IsIncoming()
{
	HRESULT hr = m_pICall->IsIncoming();

	if (FAILED(hr))
	{
		throw Exception(_T("CNmCall::IsIncoming()"), hr);
	}
	
	return (hr == S_OK);
}


void CNmCall::Reject()
{
	HRESULT hr = m_pICall->Reject();

	if (S_OK != hr)
	{
		throw Exception(_T("CNmCall::Reject()"), hr);
	}
}

CNmCall::operator INmCall *()
{
	return m_pICall;
}

void CNmCall::Connect()
{
	CComNotify::Connect(
		m_pICall, 
		IID_INmCallNotify, 
		this);
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

ULONG STDMETHODCALLTYPE CNmCall::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CNmCall::Release()
{
	return 1;
}

HRESULT STDMETHODCALLTYPE CNmCall::QueryInterface(
	REFIID riid, 
	PVOID *ppvObject)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (IUnknown *)this;
	}
	else if (riid == IID_INmCallNotify)
	{
		*ppvObject = (INmCallNotify *)this;
	}
	else
	{
		hr = E_NOINTERFACE;
		*ppvObject = NULL;
	}

	if (S_OK == hr)
	{
		AddRef();
	}

	return hr;
}

HRESULT STDMETHODCALLTYPE CNmCall::NmUI(
	CONFN /* uNotify */)
{
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CNmCall::StateChanged(
	NM_CALL_STATE /* uState */)
{
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CNmCall::Failed(
	ULONG /* uError */)
{
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CNmCall::Accepted( 
	INmConference __RPC_FAR * /* pConference */)
{
	return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
