///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMChannel.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 31/07/99 9:56 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMChannel.cpp $
// 
// 2     31/07/99 9:56 Len
// Pragma to remove warning C4711
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:25:20   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmChannel.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmChannel
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmChannel::CNmChannel(
	INmChannel *pIChannel,
	const bool bAutoConnect /* = false */)
	:	m_pIChannel(SafeAddRef(pIChannel))
{
	if (0 == m_pIChannel)
	{
		throw Exception(_T("CNmChannel::CNmChannel()"), E_INVALIDARG); 
	}

	if (bAutoConnect)
	{
		Connect();
	}
}

CNmChannel::CNmChannel(const CNmChannel &rhs)
	:	m_pIChannel(SafeAddRef(rhs.m_pIChannel))
{
	if (0 == m_pIChannel)
	{
		throw Exception(_T("CNmChannel::CNmChannel()"), E_INVALIDARG); 
	}	

	if (Connected())
	{
		Connect();
	}
}

CNmChannel::~CNmChannel()
{
	m_pIChannel->Release();
	m_pIChannel = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Implement CNmChannel
///////////////////////////////////////////////////////////////////////////////

//	EnumMember  

IEnumNmMember *CNmChannel::EnumMember()
{
	IEnumNmMember *pEnum = 0;

	HRESULT hr = m_pIChannel->EnumMember(
		&pEnum);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::EnumMember()"), hr);
	}

	return pEnum;
}


CIterateNmMember CNmChannel::IterateMembers()
{
	IEnumNmMember *pEnum = 0;

	HRESULT hr = m_pIChannel->EnumMember(
		&pEnum);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::IterateMembers()"), hr);
	}

	return CIterateNmMember(pEnum);
}

INmConference *CNmChannel::GetConference()
{
	INmConference *pConference = 0;

	HRESULT hr = m_pIChannel->GetConference(
		&pConference);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::GetConference()"), hr);
	}

	return pConference;
}

IID CNmChannel::GetInterface()
{
	IID iid;

	HRESULT hr = m_pIChannel->GetInterface(
		&iid);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::GetInterface()"), hr);
	}

	return iid;
}

ULONG CNmChannel::GetMemberCount()
{
	ULONG count;

	HRESULT hr = m_pIChannel->GetMemberCount(
		&count);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::GetMemberCount()"), hr);
	}
	
	return count;
}

ULONG CNmChannel::GetChannelType()
{
	ULONG type;

	HRESULT hr = m_pIChannel->GetNmch(
		&type);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::GetChannelType()"), hr);
	}
	
	return type;
}

bool CNmChannel::IsActive()
{
	HRESULT hr = m_pIChannel->IsActive();

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::IsActive()"), hr);
	}
	
	return (hr == S_OK);
}

bool CNmChannel::IsSameAs(const CNmChannel &rhs)
{
	HRESULT hr = m_pIChannel->IsSameAs(rhs.m_pIChannel);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::IsSameAs()"), hr);
	}
	
	return (hr == S_OK);
}

bool CNmChannel::IsSameAs(INmChannel *pChannel)
{
	HRESULT hr = m_pIChannel->IsSameAs(pChannel);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::IsSameAs()"), hr);
	}
	
	return (hr == S_OK);
}


bool CNmChannel::SetActive(bool active /* = true */)
{
	HRESULT hr = m_pIChannel->SetActive(active);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannel::SetActive()"), hr);
	}
	
	return (hr == S_OK);
}

bool CNmChannel::SetInactive()
{
	return SetActive(false);
}

CNmChannel::operator INmChannel *()
{
	return m_pIChannel;
}


void CNmChannel::Connect()
{
	CComNotify::Connect(
		m_pIChannel, 
		IID_INmChannelNotify, 
		this);
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

ULONG STDMETHODCALLTYPE CNmChannel::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CNmChannel::Release()
{
	return 1;
}

HRESULT STDMETHODCALLTYPE CNmChannel::QueryInterface(
	REFIID riid, 
	PVOID *ppvObject)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (IUnknown *)this;
	}
	else if (riid == IID_INmChannelNotify)
	{
		*ppvObject = (INmChannelNotify *)this;
	}
	else
	{
		hr = E_NOINTERFACE;
		*ppvObject = NULL;
	}

	if (S_OK == hr)
	{
		AddRef();
	}

	return hr;
}


HRESULT STDMETHODCALLTYPE CNmChannel::NmUI(
	CONFN /* uNotify */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmChannel::MemberChanged(
	NM_MEMBER_NOTIFY /* uNotify */, 
	INmMember * /* pMember */)
{
	return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
