#ifndef __NM_CHANNEL__
#define __NM_CHANNEL__
///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMChannel.hpp $
// Version        : $Revision: 1 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 23/07/99 0:00 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/NetMeeting Tools/NMChannel.hpp $
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:28:54   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "NmInterfaces.h"
#include "NmException.hpp"
#include "NmIterateMember.hpp"
#include "ComNotify.hpp"

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {
 
///////////////////////////////////////////////////////////////////////////////
// Classes defined in this file...
///////////////////////////////////////////////////////////////////////////////

class CNmChannel;

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CNmChannel
///////////////////////////////////////////////////////////////////////////////

class CNmChannel: public INmChannelNotify, private CComNotify
{
   public :

		class Exception : public CNmException
		{
			friend class CNmChannel;

			private :

				Exception(LPCTSTR pWhere, HRESULT hr) 
					: CNmException(pWhere, hr) {}

		};

      CNmChannel(
			INmChannel*pIChannel,
			const bool bAutoConnect = false);

		CNmChannel(const CNmChannel &rhs);
		
		~CNmChannel();

		// Conversion to underlying pointer...

		operator INmChannel *();


		// Event sink stuff

		void Connect();

		// Implement INmChannel

		IEnumNmMember *EnumMember();

		CIterateNmMember IterateMembers();

		INmConference *GetConference();
		
		IID GetInterface();

		ULONG GetMemberCount();
		
		ULONG GetChannelType();

		bool IsActive();

		bool IsSameAs(const CNmChannel &rhs);

		bool IsSameAs(INmChannel *pChannel);

		bool SetActive(bool active = true);

		bool SetInactive();


		// Implement INmChannelNotify
	
		// IUnknown methods
		ULONG STDMETHODCALLTYPE AddRef(void);
		ULONG STDMETHODCALLTYPE Release(void);
		HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, PVOID *ppvObj);

		HRESULT STDMETHODCALLTYPE MemberChanged(
			NM_MEMBER_NOTIFY uNotify, 
			INmMember *pMember);

		HRESULT STDMETHODCALLTYPE NmUI(
			CONFN uNotify);

	private : 

		CNmChannel &operator=(const CNmChannel &rhs);

		INmChannel *m_pIChannel;
};

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

#endif // __NM_CHANNEL__

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
