///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMChannelData.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 31/07/99 9:56 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMChannelData.cpp $
// 
// 2     31/07/99 9:56 Len
// Pragma to remove warning C4711
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:25:38   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmChannelData.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmConference
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmChannelData::CNmChannelData(
	INmChannelData *pIChannelData,
	const bool bAutoConnect /* = false */)
	:	m_pIChannelData(SafeAddRef(pIChannelData))
{
	if (!m_pIChannelData)
	{
		throw Exception(_T("CNmChannelData::CNmChannelData()"), E_INVALIDARG); 
	}

	if (bAutoConnect)
	{
		Connect();
	}
}

CNmChannelData::CNmChannelData(const CNmChannelData &rhs)
	:	m_pIChannelData(SafeAddRef(rhs.m_pIChannelData))
{
	if (!m_pIChannelData)
	{
		throw Exception(_T("CNmChannelData::CNmChannelData()"), E_INVALIDARG); 
	}	

	if (Connected())
	{
		Connect();
	}
}

CNmChannelData::CNmChannelData(
	INmChannel *pIChannel,
	const bool bAutoConnect /* = false */)
	:	m_pIChannelData((INmChannelData*)SafeQI(pIChannel, IID_INmChannelData))
{
	if (!m_pIChannelData)
	{
		throw Exception(_T("CNmChannelData::CNmChannelData()"), E_INVALIDARG); 
	}

	if (bAutoConnect)
	{
		Connect();
	}
}

CNmChannelData::~CNmChannelData()
{
	m_pIChannelData->Release();
	m_pIChannelData = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Implement CNmChannelData
///////////////////////////////////////////////////////////////////////////////

GUID CNmChannelData::GetGuid()
{
	GUID guid;

	HRESULT hr = m_pIChannelData->GetGuid(&guid);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannelData::GetGuid()"), hr);
	}

	return guid;
}

void CNmChannelData::SendData(
	INmMember *pMember,
	ULONG uSize,
	BYTE *pvBuffer,
	ULONG uOptions)
{
	HRESULT hr = m_pIChannelData->SendData(
		pMember,
		uSize,
		pvBuffer,
		uOptions);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmChannelData::SendData()"), hr);
	}
}

CNmChannelData::operator INmChannelData *()
{
	return m_pIChannelData;
}

void CNmChannelData::Connect()
{
	CComNotify::Connect(
		m_pIChannelData, 
		IID_INmChannelDataNotify, 
		this);
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

ULONG STDMETHODCALLTYPE CNmChannelData::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CNmChannelData::Release()
{
	return 1;
}

HRESULT STDMETHODCALLTYPE CNmChannelData::QueryInterface(
	REFIID riid, 
	PVOID *ppvObject)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (IUnknown *)this;
	}
	else if (riid == IID_INmChannelDataNotify)
	{
		*ppvObject = (INmChannelDataNotify *)this;
	}
	else
	{
		hr = E_NOINTERFACE;
		*ppvObject = NULL;
	}

	if (S_OK == hr)
	{
		AddRef();
	}

	return hr;
}


HRESULT STDMETHODCALLTYPE CNmChannelData::NmUI(
	CONFN /* uNotify */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmChannelData::MemberChanged(
	NM_MEMBER_NOTIFY /* uNotify */, 
	INmMember * /* pMember */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmChannelData::DataReceived(
	INmMember * /* pMember */,
	ULONG /* uSize */,
	byte * /* pvBuffer */,
	ULONG /* dwFlags */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmChannelData::DataSent(
	INmMember * /* pMember */,
	ULONG /* uSize */,
	byte * /* pvBuffer */)
{
	return S_OK;
}



///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
