///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMConference.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 31/07/99 9:56 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMConference.cpp $
// 
// 2     31/07/99 9:56 Len
// Pragma to remove warning C4711
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:25:54   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmConference.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmConference
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmConference::CNmConference(
	INmConference *pIConference,
	const bool bAutoConnect /* = false */)
	:	m_pIConference(SafeAddRef(pIConference))
{
	if (0 == m_pIConference)
	{
		throw Exception(_T("CNmConference::CNmConference()"), E_INVALIDARG); 
	}

	if (bAutoConnect)
	{
		Connect();
	}
}

CNmConference::CNmConference(const CNmConference &rhs)
	:	m_pIConference(SafeAddRef(rhs.m_pIConference))
{
	if (0 == m_pIConference)
	{
		throw Exception(_T("CNmConference::CNmConference()"), E_INVALIDARG); 
	}	

	if (Connected())
	{
		Connect();
	}
}

CNmConference::~CNmConference()
{
	m_pIConference->Release();
	m_pIConference = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Implement CNmConference
///////////////////////////////////////////////////////////////////////////////


INmChannelData *CNmConference::CreateDataChannel(REFGUID guid)
{
	INmChannelData *pIChannel = 0;

	HRESULT hr = m_pIConference->CreateDataChannel(
		&pIChannel,
		guid);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::CreateDataChannel()"), hr);
	}

	return pIChannel;
}

ULONG CNmConference::GetChannelCount()
{
	ULONG count;

	HRESULT hr = m_pIConference->GetChannelCount(&count);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::GetChannelCount()"), hr);
	}

	return count;
}

IEnumNmChannel *CNmConference::EnumChannel()
{
	IEnumNmChannel *pEnum = 0;

	HRESULT hr = m_pIConference->EnumChannel(
		&pEnum);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::EnumChannel()"), hr);
	}

	return pEnum;
}


CIterateNmChannel CNmConference::IterateChannels()
{
	IEnumNmChannel *pEnum = EnumChannel();

	CIterateNmChannel it(pEnum);

	pEnum = SafeRelease(pEnum);

	return it;
}


ULONG CNmConference::GetMemberCount()
{
	ULONG count;

	HRESULT hr = m_pIConference->GetMemberCount(&count);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::GetMemberCount()"), hr);
	}

	return count;
}

IEnumNmMember *CNmConference::EnumMember()
{
	IEnumNmMember *pEnum = 0;

	HRESULT hr = m_pIConference->EnumMember(
		&pEnum);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::EnumMember()"), hr);
	}

	return pEnum;
}


CIterateNmMember CNmConference::IterateMembers()
{
	IEnumNmMember *pEnum = EnumMember();

	CIterateNmMember it(pEnum);

	pEnum = SafeRelease(pEnum);

	return it;
}

LPTSTR CNmConference::GetName()
{
	BSTR name;

	HRESULT hr = m_pIConference->GetName(&name);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::GetName()"), hr);
	}

	COleString string(name);

	LPTSTR pName = string.DuplicateAs_tchar();
	
	SysFreeString(name);

	return pName;
}

ULONG CNmConference::GetChannelCapabilities()
{
	ULONG caps;

	HRESULT hr = m_pIConference->GetNmchCaps(&caps);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::GetChannelCapabilities()"), hr);
	}

	return caps;
}

NM_CONFERENCE_STATE CNmConference::GetState()
{
	NM_CONFERENCE_STATE state;

	HRESULT hr = m_pIConference->GetState(&state);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::GetState()"), hr);
	}

	return state;
}

void CNmConference::Host()
{
	HRESULT hr = m_pIConference->Host();

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::Host()"), hr);
	}
}

CNmConference::operator INmConference *()
{
	return m_pIConference;
}

void CNmConference::Connect()
{
	CComNotify::Connect(
		m_pIConference, 
		IID_INmConferenceNotify, 
		this);
}


void CNmConference::Leave()
{
	HRESULT hr = m_pIConference->Leave();

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConference::Leave()"), hr);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

ULONG STDMETHODCALLTYPE CNmConference::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CNmConference::Release()
{
	return 1;
}

HRESULT STDMETHODCALLTYPE CNmConference::QueryInterface(
	REFIID riid, 
	PVOID *ppvObject)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (IUnknown *)this;
	}
	else if (riid == IID_INmConferenceNotify)
	{
		*ppvObject = (INmConferenceNotify *)this;
	}
	else
	{
		hr = E_NOINTERFACE;
		*ppvObject = NULL;
	}

	if (S_OK == hr)
	{
		AddRef();
	}

	return hr;
}


HRESULT STDMETHODCALLTYPE CNmConference::NmUI(
	CONFN /* uNotify */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConference::StateChanged(
	NM_CONFERENCE_STATE /* uState */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConference::MemberChanged(
	NM_MEMBER_NOTIFY /* uNotify */, 
	INmMember * /* pMember */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConference::ChannelChanged(
	NM_CHANNEL_NOTIFY /* uNotify */, 
	INmChannel * /* pChannel */)
{
	return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
