///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMConferenceManager.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 31/07/99 9:56 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMConferenceManager.cpp $
// 
// 2     31/07/99 9:56 Len
// Pragma to remove warning C4711
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:26:26   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmConferenceManager.hpp"
#include "ComException.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmConferenceManager
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Static members
///////////////////////////////////////////////////////////////////////////////

bool CNmConferenceManager::IsNmInstalled()
{
	return E_FAIL != CheckInstalled(); 
}

HRESULT CNmConferenceManager::CheckInstalled()
{
	HRESULT hrInstalled = E_FAIL;			// No Netmeeting components are installed

	INmManager *pIManager = 0;

	CComUsesCom(COINIT_APARTMENTTHREADED);

	HRESULT hr = CoCreateInstance(
		CLSID_NmManager,
		0,
		CLSCTX_INPROC,
		IID_INmManager,
		(void**)&pIManager);

	if (SUCCEEDED(hr))
	{
		INmSysInfo *pISysInfo = 0;

		hr = pIManager->GetSysInfo(&pISysInfo);
		
		if (SUCCEEDED(hr))
		{
			hrInstalled = pISysInfo->IsInstalled();

			// E_FAIL -	Nothing installed (so how did we get this far?!)
			//
			// NM_E_NOT_INITIALIZED - One or more NetMeeting settings are not valid.  
			//
			// S_FALSE - The NetMeeting settings appear to be valid, but the 
			//           NetMeeting wizard has not run.  
			//
			// S_OK - Everything is installed properly 

			pISysInfo->Release();
		}

		pIManager->Release();
	}

	return hrInstalled;
}

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmConferenceManager::CNmConferenceManager()
	:	CComUsesCom(COINIT_APARTMENTTHREADED), // Doesnt work if we're in the MTA
		m_pIManager(0)
{
	HRESULT hr = CoCreateInstance(
		CLSID_NmManager,
		0,
		CLSCTX_INPROC,
		IID_INmManager,
		(void**)&m_pIManager);
 
	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::CNmConferenceManager() - CoCreateInstance()"), hr);
	}

	// Connect to the conference manager object

	Connect();

}

CNmConferenceManager::~CNmConferenceManager()
{
	if (m_pIManager)
	{
		m_pIManager->Release();
		m_pIManager = 0;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Implement INmManager
///////////////////////////////////////////////////////////////////////////////

void CNmConferenceManager::Initialize(
	ULONG &options, 
	ULONG &uChannelCaps)
{	
	HRESULT hr = m_pIManager->Initialize(&options, &uChannelCaps);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::Initialize()"), hr);
	}
}

INmConference *CNmConferenceManager::CreateConference(
	LPCTSTR pName, 
	//LPCTSTR pPassword,
	ULONG channelCapabilities)
{
	INmConference *pConference = 0;

	BSTR name = SysAllocString(COleString(pName));

	//BSTR password = SysAllocString(COleString(pPassword));

	HRESULT hr = m_pIManager->CreateConference(
		&pConference,
		name,
		0,//password,
		channelCapabilities);

	SysFreeString(name);
	//SysFreeString(password);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::CreateConference()"), hr);
	}

	return pConference;
}

INmCall *CNmConferenceManager::CallConference(
	NM_CALL_TYPE callType,
	NM_ADDR_TYPE addressType,
	LPCTSTR pAddress, 
	LPCTSTR pName, 
	LPCTSTR pPassword /* = _T("") */)
{
	INmCall *pCall = 0;

	BSTR addr = SysAllocString(COleString(pAddress));
	BSTR name = SysAllocString(COleString(pName));
	BSTR password = SysAllocString(COleString(pPassword));

	HRESULT hr = m_pIManager->CallConference(
		&pCall,
		callType,
		addressType,
		addr,
		name,
		0);		// Password currently not implemented

	SysFreeString(addr);
	SysFreeString(name);
	SysFreeString(password);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::CallConference()"), hr);
	}

	return pCall;
}


INmSysInfo *CNmConferenceManager::GetSysInfo()
{
	INmSysInfo *pSysInfo = 0;

	HRESULT hr = m_pIManager->GetSysInfo(&pSysInfo);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::GetSysInfo()"), hr);
	}

	return pSysInfo;
}


CNmConferenceManager::operator INmManager *()
{
	return m_pIManager;
}

void CNmConferenceManager::Connect()
{
	CComNotify::Connect(
		m_pIManager, 
		IID_INmManagerNotify, 
		this);
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

ULONG STDMETHODCALLTYPE CNmConferenceManager::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CNmConferenceManager::Release()
{
	return 1;
}

HRESULT STDMETHODCALLTYPE CNmConferenceManager::QueryInterface(
	REFIID riid, 
	PVOID *ppvObject)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (IUnknown *)this;
	}
	else if (riid == IID_INmManagerNotify)
	{
		*ppvObject = (INmManagerNotify *)this;
	}
	else
	{
		hr = E_NOINTERFACE;
		*ppvObject = NULL;
	}

	if (S_OK == hr)
	{
		AddRef();
	}

	return hr;
}

///////////////////////////////////////////////////////////////////////////////
// Implement INmConferenceNotify
///////////////////////////////////////////////////////////////////////////////

HRESULT STDMETHODCALLTYPE CNmConferenceManager::NmUI(
	CONFN /* confn */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConferenceManager::CallCreated(
	INmCall * /* pCall */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConferenceManager::ConferenceCreated(
	INmConference * /* pConference */)
{
	return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
