///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMDataPipe.cpp $
// Version        : $Revision: 5 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 12/06/01 19:18 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMDataPipe.cpp $
// 
// 5     12/06/01 19:18 Len
// Support 3.01 SR2 build without complaining about lack of testing.
// 
// 4     19/12/99 11:25 Len
// Changed TryToConnect() so that it works in situations where the
// conference has gone inactive but the data channel wasn't removed (like
// when you're hosting a meeting and everyon leaves but you...) In these
// cases we're already connected, we just need to set the appropriate
// flag...
// 
// 3     16/12/99 17:56 Len
// Take advantage of the fact that they have fixed the data channel
// reconnection bug in build 4.4.3388  of NM.
// 
// 2     28/07/99 19:47 Len
// UNICODE fix and pragma to turn off 'automatic inline expansion' message
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:26:56   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmDataPipe.hpp"
#include "NmConference.hpp"
#include "NmChannelData.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmDataPipe
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmDataPipe::CNmDataPipe()
	:	CComUsesCom(COINIT_APARTMENTTHREADED), // Doesnt work if we're in the MTA
		m_pIManager(0),
		m_pIConference(0),
		m_pIChannelData(0),
		m_bConnected(false),
      m_bForceLeaveOnUninit(false)
{

}

CNmDataPipe::CNmDataPipe(
	REFGUID guid,
	ULONG options /* = NM_INIT_NO_LAUNCH*/,
	ULONG capabilities /* = NMCH_DATA*/)
	:	CComUsesCom(COINIT_APARTMENTTHREADED), // Doesnt work if we're in the MTA
		m_pIManager(0),
		m_pIConference(0),
		m_pIChannelData(0),
		m_bConnected(false),
      m_bForceLeaveOnUninit(false)
{
	Initialise(guid, options, capabilities);
}

CNmDataPipe::~CNmDataPipe()
{
	try
	{
		Uninitialise();
	}
	catch(...)
	{
		// Destructor mustn't throw exception..
	}
}

void CNmDataPipe::Initialise(
	REFGUID guid,
	ULONG options /* = NM_INIT_NO_LAUNCH*/,
	ULONG capabilities /* = NMCH_DATA*/)
{
	m_guid = guid;

	HRESULT hr;

	hr = CoCreateInstance(
		CLSID_NmManager,
		0,
		CLSCTX_INPROC,
		IID_INmManager,
		(void**)&m_pIManager);
 
	if (FAILED(hr))
	{
		throw Exception(_T("CNmDataPipe::Initialise() - CoCreateInstance(IID_INmManager)"), hr);
	}

   // Determine which version of NetMeeting is installed and adjust some operations accordingly...

   CheckVersion();

	// Connect to the conference manager object

	ManagerNotify::Connect(
		m_pIManager, 
		IID_INmManagerNotify, 
		this);

	hr = m_pIManager->Initialize(&options, &capabilities);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmDataPipe::Initialise() - m_pIManager->Initialize()"), hr);
	}
}

void CNmDataPipe::CheckVersion()
{
	INmSysInfo *pISysInfo = 0;

	HRESULT hr = m_pIManager->GetSysInfo(&pISysInfo);
	
   if (FAILED(hr))
	{
		throw Exception(_T("CNmDataPipe::CheckVersion() - GetSysInfo"), hr);
	}

   BSTR property = 0;

	hr = pISysInfo->GetProperty(NM_SYSPROP_BUILD_VER, &property);

   // hr = S_OK then we're version 3 or later

   m_bForceLeaveOnUninit = false;

   if (hr == S_OK)
   {
      if (0 == wcsncmp(L"4.4.", property, 4))
      {
         wchar_t *pStopStringNotUsed = 0;
         
         long build = wcstol(property + 4, &pStopStringNotUsed, 10);

         if (build < 3388) 
         {
            // must force the user to leave the conference when this pipe
            // is uninitialised as there is a bug in versions of NM prior
            // to 4.4.3388 that means that we will not be able to reconnect
            // if we dont leave the conference.

            m_bForceLeaveOnUninit = true;
         }

         // The code here has been tested with all versions of netmeeting up
         // to the version that ships with XP SP 1 (build number = 4.4.3340)

         if (build > 3340)
         {
            // This is a later version that this code has been tested with, 
            // your milage may vary...

            ::MessageBox(
               0, 
               _T("This program has not been tested against this version of NetMeeting."), 
               _T("Warning"), 
               MB_ICONINFORMATION);
         }
      }
   }

   SysFreeString(property);

	pISysInfo->Release();
}

void CNmDataPipe::Uninitialise()
{

	if (m_pIChannelData)
	{
		TComNotify<INmChannelData, INmChannelDataNotify>::Disconnect();
		m_pIChannelData = SafeRelease(m_pIChannelData);

	   // NM 3 prior to 4.4.3388 doesnt like us leaving like this, 
      // we need to call leave on the conference object or we can never reconnect...

      if (m_pIConference && m_bForceLeaveOnUninit)
      {
         m_pIConference->Leave();
      }
	}

	if (m_pIConference)
	{
		TComNotify<INmConference, INmConferenceNotify>::Disconnect();
		m_pIConference = SafeRelease(m_pIConference);
	}
	
	if (m_pIManager)
	{
		ManagerNotify::Disconnect();
		m_pIManager = SafeRelease(m_pIManager);
	}
}


void CNmDataPipe::SendData(
	ULONG uSize,
	BYTE *pvBuffer,
	INmMember *pMember /* = 0*/,
	ULONG uOptions /* = 0*/)
{
	if (m_pIChannelData)
	{
		HRESULT hr = m_pIChannelData->SendData(
			pMember, 
			uSize, 
			pvBuffer, 
			uOptions);

		if (FAILED(hr))
		{
			throw Exception(_T("CNmDataPipe::SendData()"), hr);
		}
	}
	else
	{
		throw Exception(_T("CNmDataPipe::SendData()"), CO_E_OBJNOTCONNECTED); 
	}
}

void CNmDataPipe::OnError(LPCTSTR /*pMessage*/)
{

}

void CNmDataPipe::Connected()
{
	if (!m_bConnected)
	{
		m_bConnected = true;
		OnConnected();
	}
}

void CNmDataPipe::Disconnected()
{
	if (m_bConnected)
	{
		m_bConnected = false;
		OnDisconnected();
	}
}

void CNmDataPipe::OnConnected()
{
}

void CNmDataPipe::OnDisconnected()
{
}

void CNmDataPipe::OnDataReceived(			
	INmMember * /* pMember */,
	ULONG /* uSize */,
	byte * /* pvBuffer */,
	ULONG /* dwFlags */)
{
}

void CNmDataPipe::OnDataSent(
	INmMember * /* pMember */,
	ULONG /* uSize */,
	byte * /* pvBuffer */)
{
}

///////////////////////////////////////////////////////////////////////////////
// Implement IUnknown
///////////////////////////////////////////////////////////////////////////////

ULONG STDMETHODCALLTYPE CNmDataPipe::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CNmDataPipe::Release()
{
	return 1;
}

HRESULT STDMETHODCALLTYPE CNmDataPipe::QueryInterface(
	REFIID riid, 
	PVOID *ppvObject)
{
	HRESULT hr = S_OK;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (INmManagerNotify *)this;
	}
	else if (riid == IID_INmManagerNotify)
	{
		*ppvObject = (INmManagerNotify *)this;
	}
	else if (riid == IID_INmConferenceNotify)
	{
		*ppvObject = (INmConferenceNotify *)this;
	}
	else if (riid == IID_INmChannelDataNotify)
	{
		*ppvObject = (INmChannelDataNotify *)this;
	}
	else
	{
		hr = E_NOINTERFACE;
		*ppvObject = NULL;
	}

	if (S_OK == hr)
	{
		AddRef();
	}

	return hr;
}

///////////////////////////////////////////////////////////////////////////////
// Implement INmConferenceNotify, 
///////////////////////////////////////////////////////////////////////////////

HRESULT STDMETHODCALLTYPE CNmDataPipe::Manager_ConferenceCreated(
	INmConference *pConference)
{
	if (m_pIConference)
	{
		// Disconnect current sink and release old conference

		TComNotify<INmConference, INmConferenceNotify>::Disconnect();

		m_pIConference->Release();
		m_pIConference = 0;
	}
	
	// Connect to new conference

	m_pIConference = pConference;
	m_pIConference->AddRef();

	TComNotify<INmConference, INmConferenceNotify>::Connect(
		m_pIConference, 
		IID_INmConferenceNotify, 
		this);

	return S_OK;
}

void CNmDataPipe::TryToConnect()
{
   if (m_pIChannelData)
   {
		Connected();
   }
   else
   {
   	static int trying = false;

	   if (!trying)
	   {
		   OnError(_T("Trying to connect"));
		   trying = true;
		   // Create new data channel

		   HRESULT hr = m_pIConference->CreateDataChannel(
			   &m_pIChannelData,
			   m_guid);

         if (SUCCEEDED(hr))
         {
			   // Connect sink to new data channel

			   TComNotify<INmChannelData, INmChannelDataNotify>::Connect(
				   m_pIChannelData, 
				   IID_INmChannelDataNotify, 
				   this);
	   
			   Connected();
         }
         else if (hr == NM_E_CHANNEL_ALREADY_EXISTS)
         {
            // Connect to the existing channel...

            // This code doesn't work!
            try
            {
               CNmConference conf(m_pIConference);

               CIterateNmChannel it = conf.IterateChannels();

               bool bConnected = false;

               while (!bConnected && it != CIterateNmChannel::End())
               {
                  CNmChannel channel(it);

                  if (NMCH_DATA == channel.GetChannelType())
                  {
                     CNmChannelData dataChannel(channel);

                     if (m_guid == dataChannel.GetGuid())
                     {
                        m_pIChannelData = dataChannel;
                  
                        m_pIChannelData->AddRef();

                        TComNotify<INmChannelData, INmChannelDataNotify>::Connect(
                           m_pIChannelData, 
                           IID_INmChannelDataNotify, 
                           this);

                        Connected();
            
                        bConnected = true;
                     }
                  }
                  ++it;
               }

               if (!bConnected)
               {
                  OnError(_T("Failed to connect to existing channel"));
               }
            } 
            catch(CNmException &e)
            {
   			   OnError(e.GetMessage());
            }
         }
         else
         {
			   m_pIChannelData = 0;

			   // Create an exception object just to do the error code
			   // mapping for us...

			   Exception e (_T("CNmDataPipe::CreateDataChannel()"), hr);

			   OnError(e.GetMessage());
         }

		   trying = false;
	   }
   }
}

HRESULT STDMETHODCALLTYPE CNmDataPipe::Conference_StateChanged(
	NM_CONFERENCE_STATE uState)
{
	switch (uState)
	{
		case NM_CONFERENCE_ACTIVE	:
		{
			//m_bConfActive = true;
			
			TryToConnect();
		}
		break;
		
		case NM_CONFERENCE_WAITING : 

			//m_bConfActive = false;

			Disconnected();
			
		break;

		case NM_CONFERENCE_IDLE : 

			//m_bConfActive = false;

			Disconnected();
			
		break;

	}

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmDataPipe::Conference_MemberChanged(
	NM_MEMBER_NOTIFY /* uNotify */, 
	INmMember * /* pMember */)
{
	TryToConnect();

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmDataPipe::Conference_ChannelChanged(
	NM_CHANNEL_NOTIFY uNotify, 
	INmChannel *pChannel)
{
	switch (uNotify)
	{
		case NM_CHANNEL_REMOVED :

			if (m_pIChannelData)
			{
				HRESULT hr = pChannel->IsSameAs(m_pIChannelData);
				
				if (S_OK == hr)
				{
					// Disconnect and release old data channel.

					TComNotify<INmChannelData, INmChannelDataNotify>::Disconnect();

					m_pIChannelData->Release();
					m_pIChannelData = 0;
				} 
				else if (FAILED(hr))
				{
					Exception e (_T("CNmDataPipe::ChannelChanged()"), hr);
				
					//Error(e.GetMessage());
				}
			}

		break;

		default :

			TryToConnect();

		break;

	}

	return S_OK;
}


HRESULT STDMETHODCALLTYPE CNmDataPipe::Channel_DataReceived(
	INmMember *pMember,
	ULONG uSize,
	byte *pvBuffer,
	ULONG dwFlags)
{
	OnDataReceived(pMember, uSize, pvBuffer, dwFlags);
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmDataPipe::Channel_DataSent(
	INmMember *pMember,
	ULONG uSize,
	byte *pvBuffer)
{
	OnDataSent(pMember, uSize, pvBuffer);
	return S_OK;
}

void CNmDataPipe::Connect(
	LPCTSTR pAddress, 
	LPCTSTR pName, 
	LPCTSTR pPassword /* = "" */)
{
	NM_CALL_TYPE callType = NM_CALL_DEFAULT;
	NM_ADDR_TYPE addressType = NM_ADDR_UNKNOWN;

	INmCall *pCall = 0;

	BSTR addr = SysAllocString(COleString(pAddress));
	BSTR name = SysAllocString(COleString(pName));
	BSTR password = SysAllocString(COleString(pPassword));

	HRESULT hr = m_pIManager->CallConference(
		&pCall,
		callType,
		addressType,
		addr,
		name,
		0);		// Password currently not implemented

	SysFreeString(addr);
	SysFreeString(name);
	SysFreeString(password);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmDataPipe::Connect()"), hr);
	}
}

void CNmDataPipe::Disconnect()
{
	if (m_pIConference)
	{
		m_pIConference->Leave();
	}
	else
	{
		throw Exception(_T("CNmDataPipe::Disconnect()"), CO_E_OBJNOTCONNECTED);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
