#ifndef __NM_DATA_PIPE__
#define __NM_DATA_PIPE__
///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMDataPipe.hpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 16/12/99 17:57 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMDataPipe.hpp $
// 
// 2     16/12/99 17:57 Len
// Added version check stuff so we dont always have to leave the
// conference when the pipe is shut down.
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:29:54   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "NmInterfaces.h"

#include "NmException.hpp"
#include "NmMessages.hpp"
#include "TComNotify.hpp"
#include "ComUsesCom.hpp"

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {
 
///////////////////////////////////////////////////////////////////////////////
// Classes defined in this file...
///////////////////////////////////////////////////////////////////////////////

class CNmDataPipe;

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// CNmDataPipe
///////////////////////////////////////////////////////////////////////////////

class ManagerNotify : 
	public INmManagerNotify, 
	protected TComNotify<INmManager, INmManagerNotify>
{

	public :

		HRESULT STDMETHODCALLTYPE NmUI(
			CONFN confn)
		{
			Debug(_T("INmManagerNotify"), CNmMessages::NmUI(confn));
			return Manager_NmUI(confn);
		}

		HRESULT STDMETHODCALLTYPE ConferenceCreated(
			INmConference *pConference)
		{
//			Debug(_T("INmManagerNotify"), CNmMessages::ConferenceCreated(pConference));
			return Manager_ConferenceCreated(pConference);
		}

		HRESULT STDMETHODCALLTYPE CallCreated(
			INmCall *pCall)
		{
//			Debug(_T("INmManagerNotify"), CNmMessages::CallCreated(pCall));
			return Manager_CallCreated(pCall);
		}

	protected :

		virtual HRESULT STDMETHODCALLTYPE Manager_NmUI(
			CONFN /* confn */)
		{
			return S_OK;
		}

		virtual HRESULT STDMETHODCALLTYPE Manager_ConferenceCreated(
			INmConference * /* pConference */)
		{
			return S_OK;
		}

		virtual HRESULT STDMETHODCALLTYPE Manager_CallCreated(
			INmCall * /* pCall */)
		{
			return S_OK;
		}

		virtual void Debug(
			LPCTSTR /* pWho */, 
			LPCTSTR /* pWhat */)
		{
		}
};

class ConferenceNotify :
	public INmConferenceNotify, 
	public  TComNotify<INmConference, INmConferenceNotify> // should be protected
{
	public :

		HRESULT STDMETHODCALLTYPE NmUI(
			CONFN uNotify)
		{
			Debug(_T("INmConferenceNotify"), CNmMessages::NmUI(uNotify));
			return Conference_NmUI(uNotify);
		}

		HRESULT STDMETHODCALLTYPE StateChanged(
			NM_CONFERENCE_STATE uState)
		{
			Debug(_T("INmConferenceNotify"), CNmMessages::StateChanged(uState));
			return Conference_StateChanged(uState);
		}
		
		HRESULT STDMETHODCALLTYPE MemberChanged(
			NM_MEMBER_NOTIFY uNotify, 
			INmMember *pMember)
		{
			Debug(_T("INmConferenceNotify"), CNmMessages::MemberChanged(uNotify, pMember));
			return Conference_MemberChanged(uNotify, pMember);
		}
		
		HRESULT STDMETHODCALLTYPE ChannelChanged(
			NM_CHANNEL_NOTIFY uNotify, 
			INmChannel *pChannel)
		{
			Debug(_T("INmConferenceNotify"), CNmMessages::ChannelChanged(uNotify, pChannel));
			return Conference_ChannelChanged(uNotify, pChannel);
		}

	protected :

		virtual HRESULT STDMETHODCALLTYPE Conference_NmUI(
			CONFN /* uNotify */)
		{
			return S_OK;
		}

		virtual HRESULT STDMETHODCALLTYPE Conference_StateChanged(
			NM_CONFERENCE_STATE /* uState */)
		{
			return S_OK;
		}
		
		virtual HRESULT STDMETHODCALLTYPE Conference_MemberChanged(
			NM_MEMBER_NOTIFY /* uNotify */, 
			INmMember * /* pMember */)
		{
			return S_OK;
		}
		
		virtual HRESULT STDMETHODCALLTYPE Conference_ChannelChanged(
			NM_CHANNEL_NOTIFY /* uNotify */, 
			INmChannel * /* pChannel */)
		{
			return S_OK;
		}

		virtual void Debug(
			LPCTSTR /* pWho */, 
			LPCTSTR /* pWhat */)
		{
		}


};

class ChannelDataNotify :
	public INmChannelDataNotify,
	protected TComNotify<INmChannelData, INmChannelDataNotify>
{
	public :

		HRESULT STDMETHODCALLTYPE DataReceived(
			INmMember *pMember,
			ULONG uSize,
			byte *pvBuffer,
			ULONG dwFlags)
		{
			Debug(_T("INmChannelDataNotify - in"), (LPCTSTR)pvBuffer);
			return Channel_DataReceived(pMember, uSize, pvBuffer, dwFlags);
		}

		HRESULT STDMETHODCALLTYPE DataSent(
			INmMember *pMember,
			ULONG uSize,
			byte *pvBuffer)
		{
			Debug(_T("INmChannelDataNotify - out"), (LPCTSTR)pvBuffer);
			return Channel_DataSent(pMember, uSize, pvBuffer);
		}

		HRESULT STDMETHODCALLTYPE MemberChanged(
			NM_MEMBER_NOTIFY uNotify, 
			INmMember *pMember)
		{
			Debug(_T("INmChannelDataNotify"), CNmMessages::MemberChanged(uNotify, pMember));
			return Channel_MemberChanged(uNotify, pMember);
		}

		HRESULT STDMETHODCALLTYPE NmUI(
			CONFN uNotify)
		{
			Debug(_T("INmChannelDataNotify"), CNmMessages::NmUI(uNotify));
			return Channel_NmUI(uNotify);
		}

	protected :

		virtual HRESULT STDMETHODCALLTYPE Channel_DataReceived(
			INmMember * /* pMember */,
			ULONG /* uSize */,
			byte * /* pvBuffer */,
			ULONG /* dwFlags */)
		{
			return S_OK;
		}

		virtual HRESULT STDMETHODCALLTYPE Channel_DataSent(
			INmMember * /* pMember */,
			ULONG /* uSize */,
			byte * /* pvBuffer */)
		{
			return S_OK;
		}

		virtual HRESULT STDMETHODCALLTYPE Channel_MemberChanged(
			NM_MEMBER_NOTIFY /* uNotify */, 
			INmMember * /* pMember */)
		{
			return S_OK;
		}

		virtual HRESULT STDMETHODCALLTYPE Channel_NmUI(
			CONFN /* uNotify */)
		{
			return S_OK;
		}

		virtual void Debug(
			LPCTSTR /* pWho */, 
			LPCTSTR /* pWhat */)
		{
		}
};
	
class CNmDataPipe : 
	private CComUsesCom,
	private ManagerNotify,
	private ConferenceNotify,
	private ChannelDataNotify
{
   public :

		class Exception : public CNmException
		{
			friend class CNmDataPipe;

			private :

				Exception(LPCTSTR pWhere, HRESULT hr) 
					: CNmException(pWhere, hr) {}

		};

      CNmDataPipe();

      CNmDataPipe(
			REFGUID guid,
			ULONG options = NM_INIT_NO_LAUNCH,
			ULONG capabilities = NMCH_DATA);

		~CNmDataPipe();

		void Initialise(
			REFGUID guid,
			ULONG options = NM_INIT_NO_LAUNCH,
			ULONG capabilities = NMCH_DATA);

		void Uninitialise();

		void SendData(
			ULONG uSize,
			BYTE *pvBuffer,
			INmMember *pMember = 0,
			ULONG uOptions = 0);

		void Connect(
			LPCTSTR pAddress, 
			LPCTSTR pName, 
			LPCTSTR pPassword = _T("") );
	
		void Disconnect();

		virtual void OnError(LPCTSTR pMessage);
		virtual void OnConnected();
		virtual void OnDisconnected();
		virtual void OnDataReceived(			
			INmMember *pMember,
			ULONG uSize,
			byte *pvBuffer,
			ULONG dwFlags);
		virtual void OnDataSent(
			INmMember *pMember,
			ULONG uSize,
			byte *pvBuffer);
		
		// IUnknown methods
		ULONG STDMETHODCALLTYPE AddRef(void);
		ULONG STDMETHODCALLTYPE Release(void);
		HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, PVOID *ppvObj);

	private : 
	
		HRESULT STDMETHODCALLTYPE Manager_ConferenceCreated(
			INmConference *pConference);
		
		HRESULT STDMETHODCALLTYPE Conference_StateChanged(
			NM_CONFERENCE_STATE uState);
		
		HRESULT STDMETHODCALLTYPE Conference_MemberChanged(
			NM_MEMBER_NOTIFY uNotify, 
			INmMember *pMember);

		HRESULT STDMETHODCALLTYPE Conference_ChannelChanged(
			NM_CHANNEL_NOTIFY uNotify, 
			INmChannel *pChannel);

		HRESULT STDMETHODCALLTYPE Channel_DataReceived(
			INmMember *pMember,
			ULONG uSize,
			byte *pvBuffer,
			ULONG dwFlags);

		HRESULT STDMETHODCALLTYPE Channel_DataSent(
			INmMember *pMember,
			ULONG uSize,
			byte *pvBuffer);

		void TryToConnect();

		// Don't implement - restrict copying
		CNmDataPipe(const CNmDataPipe &rhs);
		CNmDataPipe &operator=(const CNmDataPipe &rhs);

		INmManager *m_pIManager;
		INmConference *m_pIConference;
		INmChannelData *m_pIChannelData;

		void Connected();
		void Disconnected();

      void CheckVersion();

		GUID m_guid;

		bool m_bConnected;

      bool m_bForceLeaveOnUninit;
};

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

#endif // __NM_DATA_PIPE__

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
