///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMMember.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 31/07/99 10:02 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/NMTools/NMMember.cpp $
// 
// 2     31/07/99 10:02 Len
// Pragma to remove warning C4711
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:27:44   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmMember.hpp"
#include "NmConference.hpp"
#include "NmChannel.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmMember
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmMember::CNmMember(INmMember *pIMember)
	:	m_pIMember(SafeAddRef(pIMember))
{
	if (0 == m_pIMember)
	{
		throw Exception(_T("CNmMember::CNmMember()"), E_INVALIDARG); 
	}
}

CNmMember::CNmMember(const CNmMember &rhs)
	:	m_pIMember(SafeAddRef(rhs.m_pIMember))
{
	if (0 == m_pIMember)
	{
		throw Exception(_T("CNmMember::CNmMember()"), E_INVALIDARG); 
	}	
}

CNmMember::~CNmMember()
{
	m_pIMember->Release();
	m_pIMember = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Implement CNmMember
///////////////////////////////////////////////////////////////////////////////

void CNmMember::Eject()
{
	HRESULT hr = m_pIMember->Eject();

	if (S_OK != hr)
	{
		throw Exception(_T("CNmMember::Eject()"), hr);
	}
}

// GetAddress()

INmConference *CNmMember::GetConference()
{
	INmConference *pConference = 0;

	HRESULT hr = m_pIMember->GetConference(
		&pConference);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::GetConference()"), hr);
	}

	return pConference;
}

ULONG CNmMember::GetID()
{
	ULONG id;

	HRESULT hr = m_pIMember->GetID(
		&id);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::GetID()"), hr);
	}

	return id;
}

LPTSTR CNmMember::GetName()
{
	BSTR name;

	HRESULT hr = m_pIMember->GetName(&name);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::GetName()"), hr);
	}

	COleString string(name);

	LPTSTR pName = string.DuplicateAs_tchar();
	
	SysFreeString(name);

	return pName;
}

ULONG CNmMember::GetChannelCapabilities()
{
	ULONG caps;

	HRESULT hr = m_pIMember->GetNmchCaps(&caps);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::GetChannelCapabilities()"), hr);
	}

	return caps;
}

ULONG CNmMember::GetNmVersion()
{
	ULONG version;

	HRESULT hr = m_pIMember->GetNmVersion(&version);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::GetNmVersion()"), hr);
	}

	return version;
}

NM_SHARE_STATE CNmMember::GetShareState()
{
	NM_SHARE_STATE state;

	HRESULT hr = m_pIMember->GetShareState(&state);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::GetShareState()"), hr);
	}

	return state;
}

bool CNmMember::IsMCU()
{
	HRESULT hr = m_pIMember->IsMCU();

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::IsMCU()"), hr);
	}
	
	return (hr == S_OK);
}

bool CNmMember::IsSelf()
{
	HRESULT hr = m_pIMember->IsSelf();

	if (FAILED(hr))
	{
		throw Exception(_T("CNmMember::IsSelf()"), hr);
	}
	
	return (hr == S_OK);
}

bool CNmMember::IsDataChannelActive()
{
	bool bActive = false;

	ULONG id;

	if (SUCCEEDED(m_pIMember->GetID(&id)) &&
		 (GetChannelCapabilities() & NMCH_DATA))
	{
		bActive = true;
/*
		// Get the conference

		INmConference *pConf = GetConference();
		CNmConference conf(pConf);
		pConf->Release();

		for (CIterateNmChannel channelIt(conf.IterateChannels());
			channelIt != CIterateNmChannel::End();
			++channelIt)
		{
			CNmChannel channel(channelIt);

			if (channel.GetChannelType() == NMCH_DATA)
			{
				for (CIterateNmMember memberIt(channel.IterateMembers());
					memberIt != CIterateNmMember::End() && !bActive;
					++memberIt)
				{
					if (CNmMember(memberIt).GetID() == GetID())
					{
						bActive = true;
					}
				}
			}
		}
*/
	}
	
	return bActive;
}

LPCTSTR CNmMember::GetFullDetails()
{
	TCHAR buffer[200];
/*
	LPCSTR pName = GetName();

	ULONG channelCapabilities = GetChannelCapabilities();

	if (
		
		NMCH_AUDIO  Audio channel.  
NMCH_DATA  Data channel.  
NMCH_FT  File Transfer channel.  
NMCH_SHARE  Application Sharing channel.  
NMCH_VIDEO  Video channel.  


	_stprintf(buffer, _T("%s: %d %d %d"), 
		pName, 
		member.IsDataChannelActive(), member.GetChannelCapabilities(), member.IsMCU());
	
	free pName;
*/
	return _tcsdup(buffer);
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
