///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMSystemInfo.cpp $
// Version        : $Revision: 1 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 23/07/99 0:00 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/NetMeeting Tools/NMSystemInfo.cpp $
// 
// 1     23/07/99 0:00 Len
// Moved to source safe
// 
//    Rev 1.0   May 13 1999 22:28:38   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1999 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "NmSystemInfo.hpp"
#include "ComUtils.hpp"
#include "OleString.hpp"

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CNmSysInfo
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmSystemInfo::CNmSystemInfo(INmSysInfo *pISysInfo)
	:	m_pISysInfo(SafeAddRef(pISysInfo))
{
	if (0 == m_pISysInfo)
	{
		throw Exception(_T("CNmSystemInfo::CNmSystemInfo()"), E_INVALIDARG); 
	}
}

CNmSystemInfo::CNmSystemInfo(const CNmSystemInfo &rhs)
	:	m_pISysInfo(SafeAddRef(rhs.m_pISysInfo))
{
	if (0 == m_pISysInfo)
	{
		throw Exception(_T("CNmSystemInfo::CNmSystemInfo()"), E_INVALIDARG); 
	}	
}

CNmSystemInfo::~CNmSystemInfo()
{
	m_pISysInfo->Release();
	m_pISysInfo = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Implement CNmSystemInfo
///////////////////////////////////////////////////////////////////////////////

bool CNmSystemInfo::GetChannelCapabilities(ULONG &channelCapabilities)
{
	HRESULT hr = m_pISysInfo->GetNmchCaps(&channelCapabilities);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmSystemInfo::GetChannelCapabilities()"), hr);
	}

	return hr == S_OK;
}


LPTSTR CNmSystemInfo::GetProperty(
	NM_SYSPROP property)
{
	BSTR value;

	HRESULT hr = m_pISysInfo->GetProperty(property, &value);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmSystemInfo::GetProperty()"), hr);
	}

	COleString string(value);

	LPTSTR pValue = string.DuplicateAs_tchar();
	
	SysFreeString(value);

	return pValue;
}


void CNmSystemInfo::SetProperty(NM_SYSPROP property, LPCTSTR pValue)
{
	BSTR value = SysAllocString(COleString(pValue));

	HRESULT hr = m_pISysInfo->SetProperty(property, value);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmSystemInfo::SetProperty()"), hr);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
