///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: EventLog.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 28/07/99 20:21 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/Win32 Tools/EventLog.cpp $
// 
// 2     28/07/99 20:21 Len
// Pragma to remove warning C4711
// 
// 1     22/07/99 23:59 Len
// Moved to source safe
// 
//    Rev 1.2   Mar 19 1999 08:02:56   len
// Removed reference to unused parameter in RegisterMessageFile() 
// to get rid of an "unused" warning...
// 
//    Rev 1.1   Jun 04 1998 00:01:04   Len
// Tidied up.
// 
//    Rev 1.0   May 18 1998 07:49:20   Len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1998 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "EventLog.hpp"
#include "Sid.hpp"
#include "Utils.hpp"    // For BOOL_to_bool

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CEventLog
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CEventLog::CEventLog(LPCTSTR pName, LPCTSTR pUNCServerName /* = 0 */)
   :  m_pName(pName), 
      m_hEvent(RegisterEventSource(pUNCServerName, m_pName))
{
   if (!m_hEvent)
   {
      throw Exception(_T("CEventLog::CEventLog()"), GetLastError());
   }
}

CEventLog::~CEventLog()
{
   DeregisterEventSource(m_hEvent);
}

///////////////////////////////////////////////////////////////////////////////
// Static registration functions
///////////////////////////////////////////////////////////////////////////////

void CEventLog::RegisterMessageFile(LPCTSTR /*pFile*/ /*= 0*/)
{

}

///////////////////////////////////////////////////////////////////////////////
// Log messages
///////////////////////////////////////////////////////////////////////////////

bool CEventLog::ReportEvent(
   WORD wType,             
   WORD wCategory,         
   DWORD dwEventID,        
   WORD wNumStrings /*= 0*/,  
   LPCTSTR *lpStrings /*= 0*/,
   DWORD dwDataSize /*= 0*/,
   LPVOID lpRawData /*= 0*/)   
{
   return BOOL_to_bool(::ReportEvent(
      m_hEvent, 
      wType, 
      wCategory, 
      dwEventID, 
      CSid(), 
      wNumStrings, 
      dwDataSize, 
      lpStrings, 
      lpRawData));
}


CEventLog::Exception::Exception(
   const LPCTSTR pWhere, 
   LONG error)
   :  CWin32Exception(pWhere, (DWORD)error)
{
   // Registry functions return errors as LONGs everything else returns them
   // as DWORDs...
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
