///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: LocalSecurityPolicy.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 28/07/99 20:21 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/Win32 Tools/LocalSecurityPolicy.cpp $
// 
// 2     28/07/99 20:21 Len
// Pragma to remove warning C4711
// 
// 1     22/07/99 23:59 Len
// Moved to source safe
// 
//    Rev 1.0   Mar 19 1999 08:04:18   len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1998 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "LocalSecurityPolicy.hpp"
#include "Sid.hpp"
#include "OleString.hpp"

#include <stdlib.h> 

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CLocalSecurityPolicy
///////////////////////////////////////////////////////////////////////////////

class CObjectAttributes : public LSA_OBJECT_ATTRIBUTES
{
	public :

		CObjectAttributes()
		{
			// er...
			memset(this, 0, sizeof(LSA_OBJECT_ATTRIBUTES));

			Length = sizeof(LSA_OBJECT_ATTRIBUTES);
		}
};

class CUnicodeString : public UNICODE_STRING
{
	public :

		CUnicodeString(LPCTSTR pString)
		{
			Buffer = COleString(pString).DuplicateAs_wchar_t();
			Length = (USHORT)wcslen(Buffer);
			MaximumLength = Length;
		}

		~CUnicodeString()
		{
			free(Buffer);
		}

};

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CLocalSecurityPolicy::CLocalSecurityPolicy(
	LPTSTR pRemoteMachine,
	ACCESS_MASK desiredAccess /* = POLICY_ALL_ACCESS */)
   :  m_hPolicy(0)
{
	CObjectAttributes objectAttributes;

	CUnicodeString remoteMachine(pRemoteMachine);

	DWORD result = LsaOpenPolicy(
		&remoteMachine,
		&objectAttributes,
		desiredAccess,
		&m_hPolicy);

	if (ERROR_SUCCESS != result)
	{
		throw Exception(_T("CLocalSecurityPolicy::CLocalSecurityPolicy() - LsaOpenPolicy()"), 
			result);
	}
}

CLocalSecurityPolicy::CLocalSecurityPolicy(
	ACCESS_MASK desiredAccess /* = POLICY_ALL_ACCESS */)
   :  m_hPolicy(0)
{
	CObjectAttributes objectAttributes;

	DWORD result = LsaOpenPolicy(
		0,										// Use local machine
		&objectAttributes,
		desiredAccess,
		&m_hPolicy);

	if (ERROR_SUCCESS != result)
	{
		throw Exception(_T("CLocalSecurityPolicy::CLocalSecurityPolicy() - LsaOpenPolicy()"), 
			result);
	}
}

CLocalSecurityPolicy::~CLocalSecurityPolicy() 
{ 
   LsaClose(m_hPolicy);
}

void CLocalSecurityPolicy::StorePrivateData(
	LPCTSTR key,
	LPCTSTR data)
{
	CUnicodeString keyString(key);
	CUnicodeString dataString(data);

	DWORD result = LsaStorePrivateData(
		m_hPolicy,
		&keyString,
		&dataString);

	if (ERROR_SUCCESS != result)
	{
		throw Exception(_T("CLocalSecurityPolicy::StorePrivateData() - LsaStorePrivateData()"), 
			result);
	}
}

void CLocalSecurityPolicy::AddAccountRights(
	LPCTSTR user,
	LPCTSTR privilege)
{
	CSid userSID(user);

	CUnicodeString privilegeString(privilege);

	PSID pSid = userSID;

	if (!LsaAddAccountRights(
		m_hPolicy,
		pSid,
		&privilegeString,
		1))
	{
		throw Exception(_T("CLocalSecurityPolicy::AddAccountRights() - LsaAddAccountRights()"), 
			GetLastError());
	}
}

///////////////////////////////////////////////////////////////////////////////
// Conversion 
///////////////////////////////////////////////////////////////////////////////

CLocalSecurityPolicy::operator HANDLE() const
{
   return m_hPolicy;
}

CLocalSecurityPolicy::Exception::Exception(
   const LPCTSTR pWhere, 
   LONG error)
   :  CWin32Exception(pWhere, (DWORD)error)
{
}


///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
