#ifndef __REGISTRY_LIST__
#define __REGISTRY_LIST__
///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: RegistryList.hpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 29/11/99 0:39 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Source/JetByteTools/Win32Tools/RegistryList.hpp $
// 
// 2     29/11/99 0:39 Len
// Now return a T* where we did return a T by value.
// 
// 1     22/07/99 23:59 Len
// Moved to source safe
// 
//    Rev 1.4   May 13 1999 22:49:26   len
// Removed default implementation, derived classes must provide item
// manipulation functionality.
// 
//    Rev 1.3   Dec 24 1998 07:04:28   len
// Turn off warning 4710
// 
//    Rev 1.2   Oct 25 1998 11:27:18   len
// Tidy up after running Lint.
// 
//    Rev 1.1   Jun 06 1998 07:44:00   Len
// Use DeleteKeyAndSubKeys so that we're sure to remove all trace of the item.
// 
//    Rev 1.0   May 25 1998 10:57:42   Len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1998 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

#include "RegistryKey.hpp"

#pragma warning(disable: 4710)   // function not expanded

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// Templates defined in this file...
///////////////////////////////////////////////////////////////////////////////

template <class T>
class TRegistryList;

///////////////////////////////////////////////////////////////////////////////
// CRegistryList
///////////////////////////////////////////////////////////////////////////////

template <class T>
class TRegistryList 
{
   public :
      
      class iterator : public CRegistryKey::SubkeyIterator
      {
         public :

            friend TRegistryList<T>;

            operator T*() const
            { 
               return m_list.ReadItem(*this); 
            }
       
         private :

            iterator(const TRegistryList<T> &list, const CRegistryKey::SubkeyIterator &it)
               :  TRegKeyIterator<class CRegistryKey::SubkeyIteratorImpl>(it), 
                  //CRegistryKey::SubkeyIterator(it),
                  m_list(list) 
            {
            }

            const TRegistryList<T> &m_list;

            // No assignment due to const member... 
            // Do not implement

            iterator operator=(const iterator &rhs);
      };

      friend iterator;

      TRegistryList(CRegistryKey key);

      virtual ~TRegistryList();

      bool HasItem(const T &item);

      void Add(const T &item);

      void Update(const T &item);

      void AddOrUpdate(const T &item);

      void Remove(const T &item);


      iterator Begin() const;

      iterator End() const;

   private :

      virtual LPCTSTR GetItemName(const T &item) = 0;

      virtual void AddItem(CRegistryKey &key, const T &item) = 0;
      
      virtual void UpdateItem(CRegistryKey &key, const T &item) = 0;

      virtual void RemoveItem(CRegistryKey &key, const T &item) = 0;

      virtual T *ReadItem(const iterator &it) const = 0;

      CRegistryKey m_hKey;
};

template <class T>
TRegistryList<T>::TRegistryList(CRegistryKey key)
   :  m_hKey(key)
{

}

template <class T>
TRegistryList<T>::~TRegistryList()
{
}

template <class T>
bool TRegistryList<T>::HasItem(const T &item)
{
   return m_hKey.HasSubkey(GetItemName(item));
}

template <class T>
void TRegistryList<T>::Add(const T &item)
{
   LPCTSTR pName = GetItemName(item);

   CRegistryKey itemKey = m_hKey.CreateKey(pName);

   AddItem(itemKey, item);
}

template <class T>
void TRegistryList<T>::Update(const T &item)
{
   LPCTSTR pName = GetItemName(item);

   CRegistryKey itemKey = m_hKey.OpenKey(pName);

   UpdateItem(itemKey, item);
}

template <class T>
void TRegistryList<T>::AddOrUpdate(const T &item)
{
   if (HasItem(item))
   {
      UpdateItem(item);
   }
   else
   {
      AddItem(item);
   }
}

template <class T>
void TRegistryList<T>::Remove(const T &item)
{
   try
   {
      LPCTSTR pName = GetItemName(item);
      
      CRegistryKey itemKey = m_hKey.OpenKey(pName);

      RemoveItem(itemKey, item);

      m_hKey.DeleteKeyAndSubkeys(pName);
   }
   catch(...)
   {
      // TOFU
   }
}

template <class T>
TRegistryList<T>::iterator TRegistryList<T>::Begin() const
{
   return iterator(*this, m_hKey.BeginSubkeyIteration());
}

template <class T>
TRegistryList<T>::iterator TRegistryList<T>::End() const
{
   return iterator(*this, m_hKey.EndSubkeyIteration());
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

#endif // __REGISTRY_LIST__

///////////////////////////////////////////////////////////////////////////////
// End of file
///////////////////////////////////////////////////////////////////////////////
