///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: Sid.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 28/07/99 20:21 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/Win32 Tools/Sid.cpp $
// 
// 2     28/07/99 20:21 Len
// Pragma to remove warning C4711
// 
// 1     22/07/99 23:59 Len
// Moved to source safe
// 
//    Rev 1.1   Mar 19 1999 08:07:28   len
// Cleaned up
// 
//    Rev 1.0   May 18 1998 07:49:26   Len
// Initial revision.
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1998 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// This software is not designed or intended for use in on-line control of
// aircraft, air traffic, aircraft navigation or aircraft communications; or in
// the design, construction, operation or maintenance of any nuclear
// facility. Licensee represents and warrants that it will not use or
// redistribute the Software for such purposes.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Include files
///////////////////////////////////////////////////////////////////////////////

#include "Sid.hpp"

#include <malloc.h>

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {

///////////////////////////////////////////////////////////////////////////////
// CSid
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CSid::CSid()
   : m_pSid(0)
{
   HANDLE hToken = INVALID_HANDLE_VALUE;

   if (!OpenThreadToken(GetCurrentThread(), TOKEN_READ, FALSE, &hToken) &&
        GetLastError() == ERROR_NO_TOKEN)
   {
      OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken);
   }
   
   if (hToken == INVALID_HANDLE_VALUE)
   {
      throw Exception(_T("CSid::GetSid() - "), ERROR_NO_TOKEN);
   }

	DWORD dwTokenSize = 0;

	TOKEN_USER *pUser = 0;

   if (!GetTokenInformation(hToken, TokenUser, NULL, 0, &dwTokenSize))
   {
		DWORD result = GetLastError();

      if (result == ERROR_INSUFFICIENT_BUFFER)
      {
         pUser = (TOKEN_USER *)malloc(dwTokenSize);
      }
      else 
      {
			throw Exception(_T("CSid::GetSid() - GetTokenInformation()"), 
				result);
		}
   }

   if (GetTokenInformation(hToken, TokenUser, pUser, dwTokenSize, &dwTokenSize))
   {
		if (pUser)
		{
			DWORD sidLength = GetLengthSid(pUser->User.Sid);

			m_pSid = (PSID)malloc(sidLength);
			memcpy(m_pSid, pUser->User.Sid, sidLength);

			free(pUser);
		}
		else
		{
			throw Exception(_T("CSid::GetSid() - GetTokenInformation()"), 
				GetLastError());
		}
   }
   else
   {
		throw Exception(_T("CSid::GetSid() - GetTokenInformation()"), 
			GetLastError());
   }
}

CSid::CSid(
	LPCTSTR pUserName, 
	LPCTSTR pRemoteMachine /* = 0 */)
	: m_pSid(0)
{
	DWORD sidSize = 0;

	TCHAR domain[256];
	DWORD domainSize = sizeof(domain);

	SID_NAME_USE snu;

	LookupAccountName(pRemoteMachine, 
		pUserName,
		m_pSid,
		&sidSize,
		domain,
		&domainSize,
		&snu);

	DWORD result = GetLastError();

	if (result != ERROR_INSUFFICIENT_BUFFER)
	{
		throw Exception(_T("CSid::CSid() - LookupAccountName() 1"), 
			result);
	}

	m_pSid = (PSID)malloc(sidSize);
	domainSize = sizeof(domain);

	if (!LookupAccountName(pRemoteMachine, 
		pUserName,
		m_pSid,
		&sidSize,
		domain,
		&domainSize,
		&snu))
	{
		free(m_pSid);

		throw Exception(_T("CSid::CSid() - LookupAccountName() 2"), 
			GetLastError());
	}
}

CSid::~CSid()
{
	free(m_pSid);
}

///////////////////////////////////////////////////////////////////////////////
// Access to the underlying SID.
///////////////////////////////////////////////////////////////////////////////

CSid::operator PSID ()
{
   return m_pSid;
}


CSid::Exception::Exception(
   const LPCTSTR pWhere, 
   LONG error)
   :  CWin32Exception(pWhere, (DWORD)error)
{
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools
///////////////////////////////////////////////////////////////////////////////

} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
